#!/bin/sh
#
# extgetvariable-testrunner.sh: automated test runner for extgetvariable-auto
# 
# extgetvariable-auto, automatic GetVariable plugin function test code
#                      for GNU Flash Player
# Copyright (C) 2015 Nutchanon Wetchasit
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# 
# Usage:
#     ./extgetvariable-testrunner.sh
# 
# Exit codes:
#     0         if all test passed
#     non-zero  if there are failed test, or tester encountered an error
# 
# Note:
#     This test requires a writable test directory, and a filesystem
#     that supports named pipe.
#

# Filenames and constants
LOGFILE=output.log
PIPE2CONTAINER=pipe2container
PIPE2PLAYER=pipe2toplayer
READTIMEOUT=5

# Test counts
TESTED=0
FAILED=0
PASSED=0

# check_equals($op1, $op2, $msg)
# Equality checker and counter
check_equals() {
	if [ "$1" = "$2" ]
	then
		echo "PASSED: $3"
		PASSED=`expr "$PASSED" + 1`
	else
		echo "FAILED: $3 (\"$1\" != \"$2\")"
		FAILED=`expr "$FAILED" + 1`
	fi
	TESTED=`expr "$TESTED" + 1`
}

# check_totals($op, $msg)
# Test count checker
check_totals() {
	check_equals "$TESTED" "$1" "$2"
}

# check_error($bool, $msg)
# Assert $bool is 0; if not, flag error in the test, and exit
check_error() {
	if [ "$1" -ne 0 ]
	then
		echo "ERROR: $2" >&2
		exit 1
	fi
}

# read_timeout($varname, $timeout)
# Read one line from standard input, with a specified timeout (in seconds)
read_timeout() {
	trap 'trap - USR1; return 142' USR1
	(sleep "$2" && kill -USR1 "$$" > /dev/null 2>&1) &
	TIMEOUTPID=$!
	read "$1"
	READERROR=$?
	kill "$TIMEOUTPID" > /dev/null 2>&1
	trap - USR1
	return $READERROR
}

# Create required named pipes
if [ \! -p "$PIPE2CONTAINER" ]
then
	mkfifo "$PIPE2CONTAINER"
	check_error "$?" "Failed to create a named pipe: $PIPE2CONTAINER"
fi
if [ \! -p "$PIPE2PLAYER" ]
then
	mkfifo "$PIPE2PLAYER"
	check_error "$?" "Failed to create a named pipe: $PIPE2PLAYER"
fi

# Open player-to-host pipe
exec 3<> "$PIPE2CONTAINER"
check_error $? "Failed to open a named pipe: $PIPE2CONTAINER"

# Open host-to-player pipe
exec 4<> "$PIPE2PLAYER"
check_error $? "Failed to open a named pipe: $PIPE2PLAYER"

# Start player
"gnash" -r 0 -vv -F 3:4 "extgetvariable-auto.swf" > "$LOGFILE" 2>&1 &
GNASHPID=$!

# Wait until the SWF code finish running, by loop-checking logfile
STARTCOUNTDOWN=$READTIMEOUT
while [ $STARTCOUNTDOWN -gt 0 ]
do
	if grep "TRACE: ENDOFTEST" "$LOGFILE" 2>&1 > /dev/null
	then
		break
	fi
	sleep 1
	STARTCOUNTDOWN=`expr $STARTCOUNTDOWN - 1`
done

[ $STARTCOUNTDOWN -ne 0 ]
check_equals $? 0 "Gnash-side code should be successfully run"

# Call string-returning GetVariable() on string variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>string_variable</string></arguments></invoke>' >&4

# Read for value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>This is a string</string>' "Gnash should return a correct value from GetVariable call on string"

# Call string-returning GetVariable() on integer variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>integer_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>9876</string>' "Gnash should return a correct value from GetVariable call on integer"

# Call string-returning GetVariable() on floating-point variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>float_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>9876.5432</string>' "Gnash should return a correct value from GetVariable call on floating point"

# Call string-returning GetVariable() on positive infinite
# floating-point variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>infinite_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>Infinity</string>' "Gnash should return a correct value from GetVariable call on infinity floating point"

# Call string-returning GetVariable() on negative infinite
# floating-point variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>neginfinite_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>-Infinity</string>' "Gnash should return a correct value from GetVariable call on negative infinity floating point"

# Call string-returning GetVariable() on non-number floating-point variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>nan_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>NaN</string>' "Gnash should return a correct value from GetVariable call on non-number floating point"

# Call string-returning GetVariable() on boolean variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>boolean_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>true</string>' "Gnash should return a correct value from GetVariable call on boolean"

# Call string-returning GetVariable() on null variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>null_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>null</string>' "Gnash should return a correct value from GetVariable call on null"

# Call string-returning GetVariable() on unassigned variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>unassigned_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>undefined</string>' "Gnash should return a correct value from GetVariable call on unassigned variable"

# Call string-returning GetVariable() on variable with undefined value
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>undefined_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>undefined</string>' "Gnash should return a correct value from GetVariable call on variable with undefined value"

# Call string-returning GetVariable() on non-existent variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>nonexistent_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<null/>' "Gnash should return a correct value from GetVariable call on non-existent variable"

# Call string-returning GetVariable() on string array variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>array_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>The,quick,brown,fox,jumps,over,the,lazy,dog</string>' "Gnash should return a correct value from GetVariable call on array variable"

# Call string-returning GetVariable() on object variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>object_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>[object Object]</string>' "Gnash should return a correct value from GetVariable call on object variable"

# Call string-returning GetVariable() on object variable
# with custom toString() method
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>object_variable_customstring</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>This is a custom Object.toString()</string>' "Gnash should return a correct value from GetVariable call on object variable with custom toString()"

# Call string-returning GetVariable() on function variable
echo '<invoke name="GetVariable" returntype="xml"><arguments><string>function_variable</string></arguments></invoke>' >&4

# Read for return value statement
read_timeout LINE $READTIMEOUT <&3
check_equals "$LINE" '<string>[type Function]</string>' "Gnash should return a correct value from GetVariable call on function variable"

# Close pipes
exec 3<&-
exec 4<&-

# Force Gnash to exit
kill $GNASHPID
wait $GNASHPID

# Check for total number of test run
check_totals "16" "There should be 16 tests run"

# Remove temporary files
rm "$PIPE2CONTAINER"
rm "$PIPE2PLAYER"

# Exit with code 0 if no failure, otherwise 1
if [ "$FAILED" -eq "0" ]
then
	exit 0
else
	exit 1
fi
